import logging
import time


LOGGER = logging.getLogger(__name__)


class TimeoutExpiredError(Exception):
    def __init__(self, value):
        super().__init__()
        self.value = value

    def __str__(self):
        return f"Timed Out: {self.value}"


class TimeoutSampler:
    """
    Samples the function output.

    This is a generator object that at first yields the output of function
    `func`. After the yield, it either raises instance of `TimeoutExpiredError` or
    sleeps `sleep` seconds.

    Yielding the output allows you to handle every value as you wish.

    Feel free to set the instance variables.
    """

    def __init__(
        self,
        wait_timeout,
        sleep,
        func,
        exceptions=None,
        exceptions_msg=None,
        print_log=True,
        **func_kwargs,
    ):
        self.wait_timeout = wait_timeout
        self.sleep = sleep
        self.func = func
        self.func_kwargs = func_kwargs
        self.exception = exceptions or Exception
        self.elapsed_time = None
        self.exceptions_msg = exceptions_msg
        self.print_log = print_log

    @property
    def _func_log(self):
        return f"Function: {self.func} Kwargs: {self.func_kwargs}"

    def __iter__(self):
        last_exp = None
        timeout_watch = TimeoutWatch(timeout=self.wait_timeout)
        if self.print_log:
            LOGGER.info(
                f"Waiting for {self.wait_timeout} seconds, retry every {self.sleep} seconds"
            )

        while timeout_watch.remaining_time() > 0:
            try:
                self.elapsed_time = self.wait_timeout - timeout_watch.remaining_time()
                yield self.func(**self.func_kwargs)
                self.elapsed_time = None
                time.sleep(self.sleep)

            except self.exception as exp:
                last_exp = exp
                self.elapsed_time = None

                time.sleep(self.sleep)

            finally:
                if self.elapsed_time and self.print_log:
                    LOGGER.info(f"Elapsed time: {self.elapsed_time}")

        raise TimeoutExpiredError(self._process_execution(exp=last_exp))

    def _process_execution(self, exp=None):
        exp_name = exp.__class__.__name__ if exp else "N/A"
        last_exception_log = f"Last exception: {exp_name}: {exp}"

        log = "{timeout}\n{func_log}\n{last_exception_log}".format(
            timeout=self.wait_timeout,
            func_log=self._func_log,
            last_exception_log=last_exception_log,
        )

        if self.exceptions_msg:
            if self.exceptions_msg not in str(exp):
                LOGGER.error(log)
                raise exp
            else:
                LOGGER.warning(f"{self.exceptions_msg}: Retrying")

        return log


class TimeoutWatch:
    """
    A time counter allowing to determine the time remaining since the start
    of a given interval
    """

    def __init__(self, timeout):
        self.timeout = timeout
        self.start_time = time.time()

    def remaining_time(self, log=None):
        """
        Return the remaining part of timeout since the object was created.
        """
        return self.start_time + self.timeout - time.time()



import logging

from ocp_resources.utils import TimeoutExpiredError, TimeoutSampler


LOGGER = logging.getLogger(__name__)


def _get_status_condition_log_message(**status_condition):
    log_msg = "Waiting For: \n"
    for status_condition_name, status_condition in status_condition.items():
        log_msg += (
            f"{status_condition_name}->{status_condition} \n"
            if status_condition
            else ""
        )

    return log_msg


class MTV:
    """
    Abstract Class for all Migration ToolKit For Virtualization (MTV) Resources:
        Provider
        Plan
        Migration
        StorageMap
        NetworkMap
    """

    def __init__(self):
        self.api = None
        self.name = None
        self.namespace = None
        self.kind = None
        self.Condition = None
        self.Status = None

        self.condition_message_ready = None
        self.condition_message_succeeded = None
        self.mapping = None
        self.source_provider_name = None
        self.source_provider_namespace = None
        self.destination_provider_name = None
        self.destination_provider_namespace = None

        if self.__class__.__name__ == "MTV":
            raise TypeError("MTV is not a Resource. Please Use one of its successors.")

    class ConditionMessage:
        PROVIDER_READY = "The provider is ready."
        NETWORK_MAP_READY = "The network map is ready."
        STORAGE_MAP_READY = "The storage map is ready."
        PLAN_READY = "The migration plan is ready."
        PLAN_SUCCEEDED = "The plan execution has SUCCEEDED."
        MIGRATION_READY = "The migration is ready."
        MIGRATION_RUNNING = "The migration is RUNNING"
        MIGRATION_SUCCEEDED = "The migration has SUCCEEDED."

    class ProviderType:
        VSPHERE = "vsphere"
        OPENSHIFT = "openshift"
        RHV = "ovirt"

    def wait_for_resource_status(
        self,
        condition_status,
        condition_type,
        condition_message=None,
        condition_reason=None,
        condition_category=None,
        wait_timeout=600,
    ):
        """
        Wait for MTV Resource Status Conditions.
        """

        LOGGER.info(
            _get_status_condition_log_message(
                condition_status=condition_status,
                condition_type=condition_type,
                condition_message=condition_message,
                condition_reason=condition_reason,
                condition_category=condition_category,
            )
        )

        samples = TimeoutSampler(
            wait_timeout=wait_timeout,
            sleep=1,
            func=self.api().get,
            field_selector=f"metadata.name=={self.name}",
            namespace=self.namespace,
        )
        last_condition = None
        try:
            for sample in samples:
                current_conditions = (
                    sample.items[0].status.get("conditions")
                    if sample.items and sample.items[0].status
                    else []
                )
                for condition in current_conditions:
                    last_condition = condition
                    valid_status_type = (
                        condition_status == condition.status
                        and condition_type == condition.type
                    )
                    valid_message = (
                        condition_message == condition.message
                        or condition_message is None
                    )
                    valid_reason = (
                        condition_reason == condition.reason or condition_reason is None
                    )
                    valid_category = (
                        condition_category == condition.category
                        or condition_category is None
                    )
                    if all(
                        [valid_status_type, valid_message, valid_reason, valid_category]
                    ):
                        return

        except TimeoutExpiredError:
            LOGGER.error(
                msg=f"Last Status Condition of {self.kind} {self.name} was: {last_condition}"
            )
            raise

    def wait_for_condition_ready(self):
        self.wait_for_resource_status(
            condition_message=self.condition_message_ready,
            condition_status=self.Condition.Status.TRUE,
            condition_type=self.Condition.READY,
        )

    def wait_for_condition_succeeded(self):
        self.wait_for_resource_status(
            condition_type=self.Status.SUCCEEDED,
            condition_message=self.condition_message_succeeded,
            condition_status=self.Condition.Status.TRUE,
        )

    @property
    def map_to_dict(self):
        return {
            "spec": {
                "map": self.mapping,
                "provider": {
                    "source": {
                        "name": self.source_provider_name,
                        "namespace": self.source_provider_namespace,
                    },
                    "destination": {
                        "name": self.destination_provider_name,
                        "namespace": self.destination_provider_namespace,
                    },
                },
            }
        }



import logging

from ocp_resources.resource import Resource


LOGGER = logging.getLogger(__name__)


class Event:
    """
    Allow read and remove K8s events.
    """

    api_version = Resource.ApiVersion.V1

    @classmethod
    def get(
        cls,
        dyn_client,
        namespace=None,
        name=None,
        label_selector=None,
        field_selector=None,
        resource_version=None,
        timeout=None,
    ):
        """
        get - retrieves K8s events.
        :param dyn_client: K8s client; mandatory
        :param namespace: event namespace; optional
        :param name: event name; optional
        :param label_selector: filter events by labels; comma separated string of key=value; optional
        :param field_selector: filter events by event fields; comma separated string of key=value; optional
        :param resource_version: filter events by their resource's version; optional
        :param timeout: timeout; optional
        :return: list of event objects

        example: reading all CSV Warning events in namespace "my-namespace", with reason of "AnEventReason"
              for event in Event.get(
                  default_client,
                  namespace="my-namespace",
                  field_selector="involvedObject.kind==ClusterServiceVersion,type==Warning,reason=AnEventReason",
                  timeout=10,
              ):
                print(event.object)
        """

        LOGGER.info("Reading events")
        LOGGER.debug(
            f"get events parameters: namespace={namespace}, name={name}, label_selector={label_selector}, "
            f"field_selector='{field_selector}', resource_version={resource_version}, timeout={timeout}"
        )

        event_listener = dyn_client.resources.get(
            api_version=cls.api_version, kind=cls.__name__
        )
        for event in event_listener.watch(
            namespace=namespace,
            name=name,
            label_selector=label_selector,
            field_selector=field_selector,
            resource_version=resource_version,
            timeout=timeout,
        ):
            yield event

    @classmethod
    def delete_events(
        cls,
        dyn_client,
        namespace=None,
        name=None,
        label_selector=None,
        field_selector=None,
        resource_version=None,
        timeout=None,
    ):
        """
        delete_events - delete K8s events. For example, to cleanup events before test, in order to not get old events in
                        the test, in order to prevent false positive test.
        :param dyn_client: K8s client; mandatory
        :param namespace: event namespace; optional
        :param name: event name; optional
        :param label_selector: filter events by labels; comma separated string of key=value; optional
        :param field_selector: filter events by event fields; comma separated string of key=value; optional
        :param resource_version: filter events by their resource's version; optional
        :param timeout: timeout; optional
        :return: list of event objects

        example: deleting all the event with a reason of "AnEventReason", from "my-namespace" namespace

        @pytest.fixture()
        def delete_events_before_test(default_client):
          Event.delete_events(default_client, namespace=my-namespace, field_selector="reason=AnEventReason")
        """
        LOGGER.info("Deleting events")
        LOGGER.debug(
            f"delete_events parameters: namespace={namespace}, name={name}, label_selector={label_selector}, "
            f"field_selector='{field_selector}', resource_version={resource_version}, timeout={timeout}"
        )
        dyn_client.resources.get(api_version=cls.api_version, kind=cls.__name__).delete(
            namespace=namespace,
            name=name,
            label_selector=label_selector,
            field_selector=field_selector,
            resource_version=resource_version,
            timeout=timeout,
        )



from ocp_resources.resource import NamespacedResource


class VirtualMachineInstancePreset(NamespacedResource):
    """
    VirtualMachineInstancePreset object.
    """

    api_group = NamespacedResource.ApiGroup.KUBEVIRT_IO

    def __init__(
        self,
        name=None,
        namespace=None,
        client=None,
        teardown=True,
        yaml_file=None,
    ):
        super().__init__(
            name=name,
            namespace=namespace,
            client=client,
            teardown=teardown,
            yaml_file=yaml_file,
        )



from ocp_resources.resource import NamespacedResource


class NetworkPolicy(NamespacedResource):
    """
    NetworkPolicy object.
    """

    api_group = NamespacedResource.ApiGroup.NETWORKING_K8S_IO



# -*- coding: utf-8 -*-

import logging

from ocp_resources.resource import NamespacedResource


LOGGER = logging.getLogger(__name__)


class PersistentVolumeClaim(NamespacedResource):
    """
    PersistentVolumeClaim object
    """

    api_version = NamespacedResource.ApiVersion.V1

    class Status(NamespacedResource.Status):
        BOUND = "Bound"
        TERMINATING = "Terminating"

    class AccessMode:
        """
        AccessMode object.
        """

        RWO = "ReadWriteOnce"
        ROX = "ReadOnlyMany"
        RWX = "ReadWriteMany"

    class VolumeMode:
        """
        VolumeMode object
        """

        BLOCK = "Block"
        FILE = "Filesystem"

    def __init__(
        self,
        name=None,
        namespace=None,
        client=None,
        storage_class=None,
        accessmodes=None,
        volume_mode=VolumeMode.FILE,
        size=None,
        hostpath_node=None,
        teardown=True,
        yaml_file=None,
    ):
        super().__init__(
            name=name,
            namespace=namespace,
            client=client,
            teardown=teardown,
            yaml_file=yaml_file,
        )
        self.accessmodes = accessmodes
        self.volume_mode = volume_mode
        self.size = size
        self.hostpath_node = hostpath_node
        self.storage_class = storage_class

    def to_dict(self):
        res = super().to_dict()
        if self.yaml_file:
            return res

        res.update(
            {
                "spec": {
                    "volumeMode": self.volume_mode,
                    "accessModes": [self.accessmodes],
                    "resources": {"requests": {"storage": self.size}},
                }
            }
        )
        """
        Hostpath-provisioner is "node aware", when using it,
        a node attribute on the claim must be introduced as follows.
        annotations:
          kubevirt.io/provisionOnNode: <specified_node_name>
        """
        if self.hostpath_node:
            res["metadata"]["annotations"] = {
                "kubevirt.io/provisionOnNode": self.hostpath_node
            }
        if self.storage_class:
            res["spec"]["storageClassName"] = self.storage_class

        return res

    def bound(self):
        """
        Check if PVC is bound

        Returns:
            bool: True if bound else False
        """
        LOGGER.info(f"Check if {self.kind} {self.name} is bound")
        return self.status == self.Status.BOUND

    @property
    def selected_node(self):
        return self.instance.metadata.annotations.get(
            "volume.kubernetes.io/selected-node"
        )



import json

from ocp_resources.resource import NamespacedResource, Resource


class Template(NamespacedResource):
    api_group = NamespacedResource.ApiGroup.TEMPLATE_OPENSHIFT_IO
    singular_name = "template"

    class Labels:
        BASE = f"{NamespacedResource.ApiGroup.TEMPLATE_KUBEVIRT_IO}/type=base"
        FLAVOR = "flavor.template.kubevirt.io"
        OS = "os.template.kubevirt.io"
        WORKLOAD = "workload.template.kubevirt.io"

    class Workload:
        DESKTOP = "desktop"
        HIGHPERFORMANCE = "highperformance"
        SERVER = "server"

    class Flavor:
        LARGE = "large"
        MEDIUM = "medium"
        SMALL = "small"
        TINY = "tiny"

    class Annotations:
        DEPRECATED = f"{NamespacedResource.ApiGroup.TEMPLATE_KUBEVIRT_IO}/deprecated"
        PROVIDER = f"{NamespacedResource.ApiGroup.TEMPLATE_KUBEVIRT_IO}/provider"
        PROVIDER_SUPPORT_LEVEL = (
            f"{NamespacedResource.ApiGroup.TEMPLATE_KUBEVIRT_IO}/provider-support-level"
        )
        PROVIDER_URL = (
            f"{NamespacedResource.ApiGroup.TEMPLATE_KUBEVIRT_IO}/provider-url"
        )

    class VMAnnotations:
        OS = f"{Resource.ApiGroup.VM_KUBEVIRT_IO}/os"
        FLAVOR = f"{Resource.ApiGroup.VM_KUBEVIRT_IO}/flavor"
        WORKLOAD = f"{Resource.ApiGroup.VM_KUBEVIRT_IO}/workload"

    def process(self, client=None, **kwargs):
        client = client or self.client
        instance_dict = self.instance.to_dict()
        params = instance_dict["parameters"]
        # filling the template parameters with given kwargs
        for param in params:
            try:
                param["value"] = kwargs[param["name"]]
            except KeyError:
                continue
        instance_dict["parameters"] = params
        # TODO: remove after fix - https://issues.redhat.com/browse/KNIP-1055 (bug 1753554)
        # For template validator to be used - template namespace needs to be updated
        instance_dict["objects"][0]["metadata"]["labels"][
            "vm.kubevirt.io/template.namespace"
        ] = instance_dict["metadata"]["namespace"]
        instance_json = json.dumps(instance_dict)
        body = json.loads(instance_json)
        response = client.request(
            method="Post",
            path=f"/apis/{self.api_version}/namespaces/openshift/processedtemplates",
            body=body,
        )
        return response.to_dict()["objects"]

    @staticmethod
    def generate_template_labels(os, workload, flavor):
        return [
            f"{Template.Labels.OS}/{os}",
            f"{Template.Labels.WORKLOAD}/{getattr(Template.Workload, workload.upper())}",
            f"{Template.Labels.FLAVOR}/{getattr(Template.Flavor, flavor.upper())}",
        ]



# -*- coding: utf-8 -*-

import logging

from ocp_resources.resource import NamespacedResource


LOGGER = logging.getLogger(__name__)


class UploadTokenRequest(NamespacedResource):
    """
    OpenShift UploadTokenRequest object.
    """

    api_group = NamespacedResource.ApiGroup.UPLOAD_CDI_KUBEVIRT_IO

    def __init__(
        self,
        name=None,
        namespace=None,
        client=None,
        pvc_name=None,
        teardown=True,
        yaml_file=None,
    ):
        super().__init__(
            name=name,
            namespace=namespace,
            client=client,
            teardown=teardown,
            yaml_file=yaml_file,
        )
        self.pvc_name = pvc_name

    def to_dict(self):
        res = super().to_dict()
        if self.yaml_file:
            return res

        res.update({"spec": {"pvcName": self.pvc_name}})
        return res



from ocp_resources.mtv import MTV
from ocp_resources.resource import NamespacedResource


class Plan(NamespacedResource, MTV):
    """
    Migration Tool for Virtualization (MTV) Plan Resource.

    Args:
        source_provider_name (str): MTV Source Provider CR name.
        source_provider_namespace (str): MTV Source Provider CR namespace.
        destination_provider_name (str): MTV Destination Provider CR name.
        destination_provider_namespace (str): MTV Destination Provider CR namespace.
        storage_map_name (str): MTV StorageMap CR name.
        storage_map_namespace (str): MTV StorageMap CR namespace.
        network_map_name (str): MTV NetworkMap CR name.
        network_map_namespace (str): MTV NetworkMap CR CR namespace.
        virtual_machines_list (list): A List of dicts, each contain
                                      the Name Or Id of the source Virtual Machines to migrate.
                                      Example: [ { "id": "vm-id-x" }, { "name": "vm-name-x" } ]
        warm_migration (bool, default: False): Warm (True) or Cold (False) migration.
    """

    api_group = NamespacedResource.ApiGroup.FORKLIFT_KONVEYOR_IO

    def __init__(
        self,
        name=None,
        namespace=None,
        source_provider_name=None,
        source_provider_namespace=None,
        destination_provider_name=None,
        destination_provider_namespace=None,
        storage_map_name=None,
        storage_map_namespace=None,
        network_map_name=None,
        network_map_namespace=None,
        virtual_machines_list=None,
        target_namespace=None,
        warm_migration=False,
        client=None,
        teardown=True,
        yaml_file=None,
    ):
        super().__init__(
            name=name,
            namespace=namespace,
            client=client,
            teardown=teardown,
            yaml_file=yaml_file,
        )
        self.source_provider_name = source_provider_name
        self.source_provider_namespace = source_provider_namespace
        self.destination_provider_name = destination_provider_name
        self.destination_provider_namespace = destination_provider_namespace
        self.storage_map_name = storage_map_name
        self.storage_map_namespace = storage_map_namespace
        self.network_map_name = network_map_name
        self.network_map_namespace = network_map_namespace
        self.virtual_machines_list = virtual_machines_list
        self.warm_migration = warm_migration
        self.target_namespace = target_namespace or self.namespace
        self.condition_message_ready = self.ConditionMessage.PLAN_READY
        self.condition_message_succeeded = self.ConditionMessage.PLAN_SUCCEEDED

    def to_dict(self):
        res = super().to_dict()
        if self.yaml_file:
            return res

        res.update(
            {
                "spec": {
                    "warm": self.warm_migration,
                    "targetNamespace": self.target_namespace,
                    "map": {
                        "storage": {
                            "name": self.storage_map_name,
                            "namespace": self.storage_map_namespace,
                        },
                        "network": {
                            "name": self.network_map_name,
                            "namespace": self.network_map_namespace,
                        },
                    },
                    "vms": self.virtual_machines_list,
                    "provider": {
                        "source": {
                            "name": self.source_provider_name,
                            "namespace": self.source_provider_namespace,
                        },
                        "destination": {
                            "name": self.destination_provider_name,
                            "namespace": self.destination_provider_namespace,
                        },
                    },
                }
            }
        )
        return res



from ocp_resources.resource import Resource


class CDI(Resource):
    """
    CDI object.
    """

    api_group = Resource.ApiGroup.CDI_KUBEVIRT_IO

    class Status(Resource.Status):
        DEPLOYING = "Deploying"
        DEPLOYED = "Deployed"



import logging

from urllib3.exceptions import ProtocolError

from ocp_resources.resource import NamespacedResource
from ocp_resources.utils import TimeoutExpiredError, TimeoutSampler


LOGGER = logging.getLogger(__name__)


class CatalogSourceConfig(NamespacedResource):
    api_group = NamespacedResource.ApiGroup.OPERATORS_COREOS_COM

    def __init__(
        self,
        name=None,
        namespace=None,
        source=None,
        target_namespace=None,
        packages=None,
        cs_display_name=None,
        cs_publisher=None,
        client=None,
        teardown=True,
        yaml_file=None,
    ):
        super().__init__(
            name=name,
            namespace=namespace,
            client=client,
            teardown=teardown,
            yaml_file=yaml_file,
        )
        self.source = source
        self.target_namespace = target_namespace
        self.packages = packages
        self.cs_display_name = cs_display_name
        self.cs_publisher = cs_publisher

    def to_dict(self):
        res = super().to_dict()
        if self.yaml_file:
            return res

        res.update(
            {
                "spec": {
                    "source": self.source,
                    "targetNamespace": self.target_namespace,
                    "packages": self.packages,
                    "csDisplayName": self.cs_display_name,
                    "csPublisher": self.cs_publisher,
                }
            }
        )

        return res

    def wait_for_csc_status(self, status, timeout=120):
        """
        Wait for CatalogSourceConfig to reach requested status.
        CatalogSourceConfig Status is found under currentPhase.phase.
        Example phase: {'message': 'The object has been successfully reconciled', 'name': 'Succeeded'}

        Raises:
            TimeoutExpiredError: If CatalogSourceConfig in not in desire status.
        """
        samples = TimeoutSampler(
            wait_timeout=timeout,
            sleep=1,
            exceptions=ProtocolError,
            func=self.api().get,
            field_selector=f"metadata.name=={self.name}",
            namespace=self.namespace,
        )
        current_status = None
        try:
            for sample in samples:
                if sample.items:
                    sample_status = sample.items[0].status
                    if sample_status:
                        current_status = sample_status.currentPhase.phase["name"]
                        if current_status == status:
                            return

        except TimeoutExpiredError:
            if current_status:
                LOGGER.error(f"Status of {self.kind} {self.name} is {current_status}")
            raise



import kubernetes
import pytest
from openshift.dynamic import DynamicClient

from ocp_resources.namespace import Namespace
from ocp_resources.pod import Pod
from ocp_resources.virtual_machine import VirtualMachine
from tests.utils import generate_yaml_from_template


@pytest.fixture(scope="session")
def client():
    return DynamicClient(client=kubernetes.config.new_client_from_config())


@pytest.fixture(scope="session")
def namespace():
    with Namespace(name="namespace-for-tests") as ns:
        yield ns


def test_get(client):
    Pod.get(dyn_client=client)


def test_create():
    with Namespace(name="test-namespace"):
        pass


def test_vm(namespace):
    name = "test-vm"
    with VirtualMachine(
        name=name, namespace=namespace.name, body=generate_yaml_from_template(name=name)
    ):
        pass



from ocp_resources.resource import TIMEOUT, Resource


class Project(Resource):
    """
    Project object.
    This is openshift's object which represents Namespace
    """

    api_group = Resource.ApiGroup.PROJECT_OPENSHIFT_IO

    class Status(Resource.Status):
        ACTIVE = "Active"


class ProjectRequest(Resource):
    """
    RequestProject object.
    Resource which adds Project and grand
    full access to user who originated this request
    """

    api_group = Resource.ApiGroup.PROJECT_OPENSHIFT_IO

    def __init__(
        self, name=None, client=None, teardown=True, timeout=TIMEOUT, yaml_file=None
    ):
        super().__init__(
            name=name,
            client=client,
            teardown=teardown,
            timeout=timeout,
            yaml_file=yaml_file,
        )

    def clean_up(self):
        Project(name=self.name).delete(wait=True)

    def client_wait_deleted(self, timeout):
        """
        client-side Wait until resource is deleted

        Args:
            timeout (int): Time to wait for the resource.

        """
        super().client_wait_deleted(timeout=timeout)



# -*- coding: utf-8 -*-

from ocp_resources.resource import Resource


class StorageClass(Resource):
    """
    StorageClass object.
    """

    api_group = Resource.ApiGroup.STORAGE_K8S_IO

    class Types:
        """
        These are names of StorageClass instances when you run `oc get sc`
        """

        LOCAL_BLOCK = "local-block"
        HOSTPATH = "hostpath-provisioner"
        CEPH_RBD = "ocs-storagecluster-ceph-rbd"
        NFS = "nfs"

    class Provisioner:
        HOSTPATH = "kubevirt.io/hostpath-provisioner"
        LOCAL_BLOCK = "kubernetes.io/no-provisioner"
        CEPH_RBD = "openshift-storage.rbd.csi.ceph.com"

    class VolumeBindingMode:
        """
        VolumeBindingMode indicates how PersistentVolumeClaims should be provisioned and bound.
        When unset, Immediate is used.
        When "Immediate", if you want to use the "node aware" hostpath-provisioner,
        ProvisionOnNode annotations should be introduced to PVC.
        Or in order to be able to use the hpp without specifying the node on the PVC,
        since CNV-2.2, hpp supports for "WaitForFirstConsumer".
        """

        # TODO: Rename to Uppercase
        Immediate = "Immediate"
        WaitForFirstConsumer = "WaitForFirstConsumer"

    class Annotations:
        IS_DEFAULT_CLASS = (
            f"{Resource.ApiGroup.STORAGECLASS_KUBERNETES_IO}/is-default-class"
        )



# -*- coding: utf-8 -*-

from ocp_resources.resource import Resource


class HostPathProvisioner(Resource):
    """
    HostPathProvisioner Custom Resource Object.
    """

    api_group = Resource.ApiGroup.HOSTPATHPROVISIONER_KUBEVIRT_IO

    class Name:
        HOSTPATH_PROVISIONER = "hostpath-provisioner"

    def __init__(
        self,
        name=None,
        path=None,
        image_pull_policy=None,
        client=None,
        teardown=True,
        yaml_file=None,
    ):
        super().__init__(
            name=name, client=client, teardown=teardown, yaml_file=yaml_file
        )
        self.path = path
        self.image_pull_policy = image_pull_policy

    def to_dict(self):
        res = super().to_dict()
        if self.yaml_file:
            return res

        spec = res.setdefault("spec", {})
        path_config = spec.setdefault("pathConfig", {})
        if self.path:
            path_config["path"] = self.path
        if self.image_pull_policy:
            spec["imagePullPolicy"] = self.image_pull_policy
        return res

    @property
    def volume_path(self):
        return self.instance.spec.pathConfig.path



# -*- coding: utf-8 -*-

from ocp_resources.resource import NamespacedResource


class RoleBinding(NamespacedResource):
    """
    RoleBinding object
    """

    api_group = NamespacedResource.ApiGroup.RBAC_AUTHORIZATION_K8S_IO

    def __init__(
        self,
        name=None,
        namespace=None,
        client=None,
        subjects_kind=None,
        subjects_name=None,
        subjects_namespace=None,
        subjects_api_group=None,
        role_ref_kind=None,
        role_ref_name=None,
        teardown=True,
        yaml_file=None,
    ):

        super().__init__(
            name=name,
            namespace=namespace,
            client=client,
            teardown=teardown,
            yaml_file=yaml_file,
        )
        self.subjects_kind = subjects_kind
        self.subjects_name = subjects_name
        self.subjects_namespace = subjects_namespace
        self.subjects_api_group = subjects_api_group
        self.role_ref_kind = role_ref_kind
        self.role_ref_name = role_ref_name

    def to_dict(self):
        res = super().to_dict()
        if self.yaml_file:
            return res

        subjects = {}
        if self.subjects_kind:
            subjects["kind"] = self.subjects_kind
        if self.subjects_name:
            subjects["name"] = self.subjects_name
        if self.subjects_namespace:
            subjects["namespace"] = self.subjects_namespace
        if self.subjects_api_group:
            subjects["apiGroup"] = self.subjects_api_group
        if subjects:
            res["subjects"] = [subjects]

        roleref = {}
        if self.role_ref_kind:
            roleref["kind"] = self.role_ref_kind
        if self.role_ref_name:
            roleref["name"] = self.role_ref_name
        if roleref:
            roleref["apiGroup"] = self.api_group
            res["roleRef"] = roleref
        return res



from ocp_resources.resource import NamespacedResource


class SriovNetwork(NamespacedResource):
    """
    SriovNetwork object.
    """

    api_group = NamespacedResource.ApiGroup.SRIOVNETWORK_OPENSHIFT_IO

    def __init__(
        self,
        name=None,
        namespace=None,
        network_namespace=None,
        client=None,
        resource_name=None,
        vlan=None,
        ipam=None,
        teardown=True,
        yaml_file=None,
    ):
        super().__init__(
            name=name,
            namespace=namespace,
            client=client,
            teardown=teardown,
            yaml_file=yaml_file,
        )
        self.network_namespace = network_namespace
        self.resource_name = resource_name
        self.vlan = vlan
        self.ipam = ipam

    def to_dict(self):
        res = super().to_dict()
        if self.yaml_file:
            return res

        res["spec"] = {
            "ipam": self.ipam or "{}\n",
            "networkNamespace": self.network_namespace,
            "resourceName": self.resource_name,
        }
        if self.vlan:
            res["spec"]["vlan"] = self.vlan
        return res



from ocp_resources.resource import Resource


class SecurityContextConstraints(Resource):
    """
    Security Context Constraints object.
    """

    api_group = Resource.ApiGroup.SECURITY_OPENSHIFT_IO



# -*- coding: utf-8 -*-
import logging

from urllib3.exceptions import ProtocolError

from ocp_resources.resource import TIMEOUT, NamespacedResource
from ocp_resources.utils import TimeoutSampler


LOGGER = logging.getLogger(__name__)


class Deployment(NamespacedResource):
    """
    OpenShift Deployment object.
    """

    api_group = NamespacedResource.ApiGroup.APPS

    def scale_replicas(self, replica_count=int):
        """
        Update replicas in deployment.

        Args:
            replica_count (int): Number of replicas.

        Returns:
            Deployment is updated successfully
        """
        body = super().to_dict()
        body.update({"spec": {"replicas": replica_count}})

        LOGGER.info(f"Set deployment replicas: {replica_count}")
        return self.update(resource_dict=body)

    def wait_for_replicas(self, deployed=True, timeout=TIMEOUT):
        """
        Wait until all replicas are updated.

        Args:
            deployed (bool): True for replicas deployed, False for no replicas.
            timeout (int): Time to wait for the deployment.

        Raises:
            TimeoutExpiredError: If not availableReplicas is equal to replicas.
        """
        LOGGER.info(f"Wait for {self.kind} {self.name} to be deployed: {deployed}")
        samples = TimeoutSampler(
            wait_timeout=timeout,
            sleep=1,
            exceptions=ProtocolError,
            func=self.api().get,
            field_selector=f"metadata.name=={self.name}",
        )
        for sample in samples:
            if sample.items:
                status = sample.items[0].status
                if deployed:
                    if status.replicas == status.availableReplicas:
                        return
                else:
                    if not status.availableReplicas:
                        return



# -*- coding: utf-8 -*-

from ocp_resources.resource import Resource


class PersistentVolume(Resource):
    """
    PersistentVolume object
    """

    api_version = Resource.ApiVersion.V1

    class Status(Resource.Status):
        RELEASED = "Released"

    @property
    def max_available_pvs(self):
        """
        Returns the maximum number (int) of PV's which are in 'Available' state
        """
        return len(
            [pv for pv in self.api().get()["items"] if pv.status.phase == "Available"]
        )
